// Code generated by "go run err_generate.go"; DO NOT EDIT.

package cookie

import (
	"errors"
	"fmt"
	"log/slog"

	"codeberg.org/miekg/dns/cmd/atomdns/internal/dnslog"
)

func (c *Cookie) Err(err error) error { return fmt.Errorf("%s: %w", c.Key(), err) }

func (c *Cookie) Errf(err string) error { return fmt.Errorf("%s: %w", c.Key(), errors.New(err)) }

func Err(err error) slog.Attr { return dnslog.Error(err) }

func (c *Cookie) Key() string { return "cookie" }

var log = func() *slog.Logger { return slog.Default().With("handler", "cookie") }
